/*
Author : Benot Vogel -- BASED ON A 'BLANK' PLUGIN FROM THE ISADORA 1.3 SDK PROJECT WIZARD
Date : 09/03/2014
*/
// ===========================================================================
//	Isadora Demo Plugin			   2003 Mark F. Coniglio. All rights reserved.
// ===========================================================================
//
//	IMPORTANT: This source code ("the software") is supplied to you in
//	consideration of your agreement to the following terms. If you do not
//	agree to the terms, do not install, use, modify or redistribute the
//	software.
//
//	Mark Coniglio (dba TroikaTronix) grants you a personal, non exclusive
//	license to use, reproduce, modify this software with and to redistribute it,
//	with or without modifications, in source and/or binary form. Except as
//	expressly stated in this license, no other rights are granted, express
//	or implied, to you by TroikaTronix.
//
//	This software is provided on an "AS IS" basis. TroikaTronix makes no
//	warranties, express or implied, including without limitation the implied
//	warranties of non-infringement, merchantability, and fitness for a 
//	particular purpurse, regarding this software or its use and operation
//	alone or in combination with your products.
//
//	In no event shall TroikaTronix be liable for any special, indirect, incidental,
//	or consequential damages arising in any way out of the use, reproduction,
//	modification and/or distribution of this software.
//
// ===========================================================================
//
// CUSTOMIZING THIS SOURCE CODE
// To customize this file, search for the text ###. All of the places where
// you will need to customize the file are marked with this pattern of 
// characters.
//
// ABOUT IMAGE BUFFER MAPS:
//
// The ImageBufferMap structure, and its accompanying functions,
// exists as a convenience to those writing video processing plugins.
//
// Basically, an image buffer contains an arbitrary number of input and
// output buffers (in the form of ImageBuffers). The ImageBufferMap code
// will automatically create intermediary buffers if needed, so that the
// size and depth of the source image buffers sent to your callback are
// the same for all buffers.
// 
// Typically, the ImageBufferMap is created in your CreateActor function,
// and dispose in the DiposeActor function.

// ---------------------------------------------------------------------------------
// INCLUDES
// ---------------------------------------------------------------------------------

#include "IsadoraTypes.h"
#include "IsadoraCallbacks.h"
#include "ImageBufferUtil.h"
#include "PluginDrawUtil.h"

// STANDARD INCLUDES
#include <string.h>
#include <stdio.h>


#include "MyIncludeFile.h"


// ---------------------------------------------------------------------------------
// MacOS Specific
// ---------------------------------------------------------------------------------
#if TARGET_OS_MAC
#define EXPORT_
#endif

// ---------------------------------------------------------------------------------
// Win32  Specific
// ---------------------------------------------------------------------------------
#if TARGET_OS_WIN32

	#include <windows.h>
	
	#define EXPORT_ __declspec(dllexport)
	
	#ifdef __cplusplus
	extern "C" {
	#endif

	BOOL WINAPI DllMain ( HINSTANCE hInst, DWORD wDataSeg, LPVOID lpvReserved );

	#ifdef __cplusplus
	}
	#endif

	BOOL WINAPI DllMain (
		HINSTANCE	/* hInst */,
		DWORD		wDataSeg,
		LPVOID		/* lpvReserved */)
	{
	switch(wDataSeg) {
	
	case DLL_PROCESS_ATTACH:
		return 1;
		break;
	case DLL_PROCESS_DETACH:
		break;
		
	default:
		return 1;
		break;
	}
	return 0;
	}

#endif

// ---------------------------------------------------------------------------------
//	Exported Function Definitions
// ---------------------------------------------------------------------------------

#ifdef __cplusplus
extern "C" {
#endif

EXPORT_ void GetActorInfo(void* inParam, ActorInfo* outActorParams);

#ifdef __cplusplus
}
#endif

// ---------------------------------------------------------------------------------
//	FORWARD DECLARTIONS
// ---------------------------------------------------------------------------------
static void
ReceiveMessage(
	IsadoraParameters*	ip,
	MessageMask			inMessageMask,
	PortIndex			inPortIndex1,
	const MsgData*		inData,
	UInt32				inLen,
	long				inRefCon);

// ---------------------------------------------------------------------------------
// GLOBAL VARIABLES
// ---------------------------------------------------------------------------------
// ### Declare global variables, common to all instantiations of this plugin here

// Example: static int gMyGlobalVariable = 5;

// ---------------------------------------------------------------------------------
// PluginInfo struct
// ---------------------------------------------------------------------------------
// ### This structure neeeds to contain all variables used by your plugin. Memory for
// this struct is allocated during the CreateActor function, and disposed during
// the DisposeActor function, and is private to each copy of the plugin.
//
// If your plugin needs global data, declare them as static variables within this
// file. Any static variable will be global to all instantiations of the plugin.

typedef struct {

	ActorInfo*				mActorInfoPtr;		// our ActorInfo Pointer - set during create actor function
	//MessageReceiverRef		mMessageReceiver;	// pointer to our message receiver reference
	
	MediaCode				mMediaType; // the media type as defined in IsadoraTypes.h (search 'MediaCode')
	String*					mName;	// the string to look for in scene names
	bool					mDelimited; // use a delimiter char? true/false
	int						mDelimiter;	// delimiter char value



} PluginInfo;

// A handy macro for casting the mActorDataPtr to PluginInfo*
#if __cplusplus
#define	GetPluginInfo_(actorDataPtr)		static_cast<PluginInfo*>((actorDataPtr)->mActorDataPtr);
#else
#define	GetPluginInfo_(actorDataPtr)		(PluginInfo*)((actorDataPtr)->mActorDataPtr);
#endif

// ---------------------------------------------------------------------------------
//	Constants
// ---------------------------------------------------------------------------------
//	Defines various constants used throughout the plugin.

// ### GROUP ID
// Define the group under which this plugin will be displayed in the Isadora interface.
// These are defined under "Actor Types" in IsadoraTypes.h

static const OSType	kActorClass 	= kGroupControl;

// ### PLUGIN IN
// Define the plugin's unique four character identifier. Contact TroikaTronix to
// obtain a unique four character code if you want to ensure that someone else
// has not developed a plugin with the same code. Note that TroikaTronix reserves
// all plugin codes that begin with an underline, an at-sign, and a pound sign
// (e.g., '_', '@', and '#'.)

static const OSType	kActorID		= FOUR_CHAR_CODE('uKzd');

// ### ACTOR NAME
// The name of the actor. This is the name that will be shown in the User Interface.

static const char* kActorName		= "GetBinNumberByName";

// ### PROPERTY DEFINITION STRING
// The property string. This string determines the inputs and outputs for your plugin.
// See the IsadoraCallbacks.h under the heading "PROPERTY DEFINITION STRING" for the
// meaning ofthese codes. (The IsadoraCallbacks.h header can be seen by opening up
// the IzzySDK Framework while in the Files view.)
//
// IMPORTANT: You cannot use spaces in the property name. Instead, use underscores (_)
// where you want to have a space.
//
// Note that each line ends with a carriage return (\r), and that only the last line of
// the bunch ends with a semicolon. This means that what you see below is one long
// null-terminated c-string, with the individual lines separated by carriage returns.

static const char* sPropertyDefinitionString =

// INPUT PROPERTY DEFINITIONS
//	TYPE 	PROPERTY NAME	ID		DATATYPE	DISPLAY FMT			MIN		MAX		INIT VALUE
	"INPROP media_type		mtyp	int			propdef				1		5		1\r"
	"INPROP name			nam		string		text				*		*		\r"
	"INPROP delimited		delf	bool		onoff				0		1		1\r"
	"INPROP delimiter		del 	int			char				32		126		58\r" // char(58) = ':'
	"INPROP	trigger			itrg	bool		trig				0		1		0\r"

// OUTPUT PROPERTY DEFINITIONS
//	TYPE 	 PROPERTY NAME	ID		DATATYPE	DISPLAY FMT			MIN		MAX		INIT VALUE
	"OUTPROP bin_number		bnum	int			number				1		*		0\r"
	"OUTPROP bin_found		otrg	bool		trig				0		1		0\r";

// ### Property Index Constants
// Properties are referenced by a one-based index. The first input property will
// be 1, the second 2, etc. Similarly, the first output property starts at 1.
// You whould have one constant for each input and output property defined in the 
// property definition string.

enum
{
	kInputMediaType = 1,
	kInputName,
	kInputDelimited,
	kInputDelimiter,
	kInputTrigger,
	
	kOutputBinNumber = 1,
	kOutputBinFound
};


// ---------------------
//	Help String
// ---------------------
// ### Help Strings
//
// The first help string is for the actor in general. This followed by help strings
// for all of the inputs, and then by the help strings for all of the outputs. These
// should be given in the order that they are defined in the Property Definition
// String above.
//
// In all, the total number of help strings should be (num inputs + num outputs + 1)
//
// Note that each string is followed by a comma -- it is a common mistake to forget the
// comma which results in the two strings being concatenated into one.

const char* sHelpStrings[] =
{
	"This actor looks for the 1st matching media bin name (starting from the beginning of the media bin list), whose name begins with the specified text, "
		"and displays its bin index within this media type.\n"
		"It simply does not update its outputs if no matching bin is found.\n\n"
		"This actor is compatible with the Get Media Index actor.",

	"The type of media bin to look for. Inside a user actor, the same user input can be linked to this property and to the media type property of Get Media Index at the same time.",
	
	"The text to look for in bin names. A match is defined as the "
		"search text matching the first characters of a bin name. If this is true, subsequent characters are ignored.\n\n"
		"Strange behaviours will occur if this property is empty or contains the delimiter character in it.",

	"If ON, a match requires the NAME to be found at the beginning of a bin name, "
		"followed by the delimiter character. Subsequent characters are ignored. "
		"\nThis is useful when receiving MIDI Show Control, so that, for example, "
		"Cue 10 does not erroneously match a bin labeled 100.",

	"The delimiter character to be used in bin names.\n\nList of accepted characters:\n"
		"space + !\"#$%&'()*+,-./\n"
		":;<=>?@[\\]^_`{|}~.\n"
		"0123456789\n"
		"ABCDEFGHIJKLMNOPQRSTUVWXYZ\n"
		"abcdefghijklmnopqrstuvwxyz\n",

	"Use this trigger to start looking for a matching bin name.",

	"The index of the matching bin within the list of bins of given media type (this means that bins of other media types don't interfere in this count).\n"
		"The 1st bin number is 1.",

	"Triggered when a matching bin is found."
};

// ---------------------------------------------------------------------------------
//		 CreateActor
// ---------------------------------------------------------------------------------
// Called once, prior to the first activation of an actor in its Scene. The
// corresponding DisposeActor actor function will not be called until the file
// owning this actor is closed, or the actor is destroyed as a result of being
// cut or deleted.

static void
CreateActor(
	IsadoraParameters*	ip,	
	ActorInfo*			ioActorInfo)		// pointer to this actor's ActorInfo struct - unique to each instance of an actor
{
	// creat the PluginInfo struct - initializing it to all zeroes
	PluginInfo* info = (PluginInfo*) IzzyMallocClear_(ip, sizeof(PluginInfo));
	PluginAssert_(ip, info != nil);
	
	ioActorInfo->mActorDataPtr = info;
	info->mActorInfoPtr = ioActorInfo;

	// ### allocation and initialization of private member variables
	
}

// ---------------------------------------------------------------------------------
//		 DisposeActor
// ---------------------------------------------------------------------------------
// Called when the file owning this actor is closed, or when the actor is destroyed
// as a result of its being cut or deleted.
//
static void
DisposeActor(
	IsadoraParameters*	ip,
	ActorInfo*			ioActorInfo)		// pointer to this actor's ActorInfo struct - unique to each instance of an actor
{
	PluginInfo* info = GetPluginInfo_(ioActorInfo);
	PluginAssert_(ip, info != nil);
	
	// ### destruction of private member variables
	

	// destroy the PluginInfo struct allocated with IzzyMallocClear_ the CreateActor function
	PluginAssert_(ip, ioActorInfo->mActorDataPtr != nil);
	IzzyFree_(ip, ioActorInfo->mActorDataPtr);
}

// ---------------------------------------------------------------------------------
//		 ActivateActor
// ---------------------------------------------------------------------------------
//	Called when the scene that owns this actor is activated or deactivated. The
//	inActivate flag will be true when the scene is activated, false when deactivated.
//
static void
ActivateActor(
	IsadoraParameters*	ip,
	ActorInfo*			inActorInfo,		// pointer to this actor's ActorInfo struct - unique to each instance of an actor
	Boolean				inActivate)			// true when actor is becoming active, false otherwise.
{
	PluginInfo* info = GetPluginInfo_(inActorInfo);
	
	// ------------------------
	// ACTIVATE
	// ------------------------
	
	if (inActivate) {
	
		// Isadora passes various messages to plugins that request them.
		// These include Mouse Moved messages, Key Down/Key Up messages,
		// Video Frame Clock messages, etc. The complete list can be found
		// in the enumeration in MessageReceiverCommon.h
		
		// You ask Isadora for these messages by calling CreateMessageReceiver_
		// with a pointer to your function, and the message types you would
		// like to receive. (These are bitmapped flags, so you can combine as
		// many as you like: kWantKeyDown | kWantKeyDown for instance.)
		
		// Here we request that our ReceiveMessage function is called
		// whenever the Isadora New Video Frame message is sent,
		// which happens periodically, 30 times per second. We set the ref
		// con to our ActorInfo ptr so that we can access that information
		// from ReceiveMessage callback.
		
		//MessageReceiveFunction* msgRcvFunc = ReceiveMessage;
		//
		//// if the "bypass" flag is off, then we want to receive messages
		//if (info->mBypass == false) {
		//	
		//	// we should not already have a message receiver
		//	PluginAssert_(ip, info->mMessageReceiver == nil);
		//	
		//	// create a message receiver that will be notified of
		//	// video frame ticks
		//	info->mMessageReceiver = CreateMessageReceiver_(
		//		ip,
		//		msgRcvFunc,
		//		0,
		//		kWantVideoFrameTick,
		//		(long) inActorInfo);
		//}
		//
		//// set the needs draw flag so that we will be drawn as soon
		//// as possible
		//info->mNeedsDraw = true;
	
	// ------------------------
	// DEACTIVATE
	// ------------------------
	
	} else {
	
		//// dispose our message receiver when we are deactivated.
		//if (info->mMessageReceiver != nil) {
		//	DisposeMessageReceiver_(ip, info->mMessageReceiver);
		//	info->mMessageReceiver = nil;
		//	info->mNeedsDraw |= true;
		//}
		//
		//// ### dispose any data that you don't need when 
		//// you are not active.
		//DisposeOwnedImageBuffers(ip, &info->mImageBufferMap);
		//ClearSourceBuffers(ip, &info->mImageBufferMap);
	}
}

// ---------------------------------------------------------------------------------
//		 GetParameterString
// ---------------------------------------------------------------------------------
//	Returns the property definition string. Called when an instance of the actor
//	needs to be instantiated.

static const char*
GetParameterString(
	IsadoraParameters*	/* ip */,
	ActorInfo*			/* inActorInfo */)
{
	return sPropertyDefinitionString;
}

// ---------------------------------------------------------------------------------
//		 GetHelpString
// ---------------------------------------------------------------------------------
//	Returns the help string for a particular property. If you have a fixed number of
//	input and output properties, it is best to use the PropertyTypeAndIndexToHelpIndex_
//	function to determine the correct help string to return.

static void
GetHelpString(
	IsadoraParameters*	ip,
	ActorInfo*			inActorInfo,
	PropertyType		inPropertyType,			// kPropertyTypeInvalid when requesting help for the actor
												// or kInputProperty or kOutputProperty when requesting help for a specific property
	PropertyIndex		inPropertyIndex1,		// the one-based index of the property (when inPropertyType is not kPropertyTypeInvalid)
	char*				outParamaterString,		// receives the help string
	UInt32				inMaxCharacters)		// size of the outParamaterString buffer
{
	const char* helpstr = nil;
	
	// The PropertyTypeAndIndexToHelpIndex_ converts the inPropertyType and
	// inPropertyIndex1 parameters to determine the zero-based index into
	// your list of help strings.
	UInt32 index1 = PropertyTypeAndIndexToHelpIndex_(ip, inActorInfo, inPropertyType, inPropertyIndex1);
	
	// get the help string
	helpstr = sHelpStrings[index1];
	
	// copy it to the output string
	strncpy(outParamaterString, helpstr, inMaxCharacters);
}
	




// ---------------------------------------------------------------------------------
//	###	 PropertyValueToString
// ---------------------------------------------------------------------------------
//	### This function defines custom display strings for media type input
//	### since we declared this parameter as "PROPDEF" type

static Boolean
PropertyValueToString(
	IsadoraParameters*	ip,
	ActorInfo*			inActorInfo,
	PropertyType		inPropertyType,				// property type
	PropertyIndex		inPropertyIndex1,			// the one-based index of the property than changed values
	ValuePtr			inValue,					// the property's current value
	char*				outString)					// string to display (max length of 256 characters)
{

	if (inPropertyType == kInputProperty) // input parameters only
	{

		if (inPropertyIndex1 == kInputMediaType) // 'media type' input property
		{
			switch (inValue->u.ivalue)
			{
			case MEDIA_TYPE_VIDEO:

				strcpy (outString,"video");
				return true; // update needed (I suppose that's what the Boolean return value is)

				break;
			case MEDIA_TYPE_AUDIO:

				strcpy (outString,"audio");
				return true; // update needed (I suppose that's what the Boolean return value is)

				break;
			case MEDIA_TYPE_MIDI:

				strcpy (outString,"midi");
				return true; // update needed (I suppose that's what the Boolean return value is)

				break;
			case MEDIA_TYPE_PICTURE:

				strcpy (outString,"picture");
				return true; // update needed (I suppose that's what the Boolean return value is)

				break;
			case MEDIA_TYPE_3D:

				strcpy (outString,"3D");
				return true; // update needed (I suppose that's what the Boolean return value is)

				break;

			}
		}
	}

	return false; // no update needed (I suppose that's what the Boolean return value is)

}









// ---------------------------------------------------------------------------------
//		REMOVECHARINSTRING FUNCTION
// ---------------------------------------------------------------------------------
//	This function removes all occurrences of a char parameter in a char* parameter.
//

static void removeCharInString (char c, char* s)
{
	char* pRead = s;
	char* pWrite = s;

	while(*pRead != '\0') // while not end of string
	{
		if (*pRead != c)
		{
			*pWrite = *pRead;
			pWrite++; // advance one step further only if read char differs from c

		}

		pRead++; // always read next char

		

	}
	*pWrite = '\0'; // finally, null-terminate the string

}





// ---------------------------------------------------------------------------------
//		EXTRACTFIRSTPART FUNCTION
// ---------------------------------------------------------------------------------
//	Returns a string containing every character of a string (str parameter) until some char (delimiter parameter) is found.
//	If no delimiter is found, the function returns str as is.
//	The returned string must be freed afterwards.
//

static char* ExtractFirstPart(char *str, char delim)
{
   size_t len;
   char *new_str;
   char *delim_pos = strchr(str, delim);

   /* new string is the length from the start of the old string to the
    * delimiter, or if it doesn't exist, a copy of the whole string */
   if (delim_pos == NULL)
      return strdup(str);

   len = delim_pos - str;
   new_str = (char*)malloc(len + 1);
   memcpy(new_str, str, len);
   new_str[len] = '\0'; /* null-terminate the new string */

   return new_str;
}






// ---------------------------------------------------------------------------------
//		FINDNAMEPARAMETERINBINNAME FUNCTION
// ---------------------------------------------------------------------------------
//	This function returns true if its inputName parameter and the beginning of the binName parameter are the same, else it returns false.
//	- if delimited == false, the beginning of inputName must match the beginning of binName
//	- if delimited == true, the beginning of inputName must match all characters of binName until the delimiter char is found (delimiter excluded)


static bool
FindNameParameterInBinName(
	char*	inputName,
	char*	binName,
	bool	delimited,
	char	delimiter
	)
{	
	

	if (delimited == false) // If delimited property is off
	{
		// Don't use the delimiter, the bin name only has to begin with the input name property
		return (strncmp(binName, inputName, strlen(inputName)) == 0); // if beginning of bin name matches the input name property: return true, else return false

	}else{ // If delimited property is on

		
		// If the delimiter exists in bin name, extract the beginning of the bin name (if not, use entire bin name as a base for comparison)
		char* strFirstPart = ExtractFirstPart(binName, delimiter);

		if (strcmp(inputName, strFirstPart) == 0) // if input name parameter and extracted first part of bin name (or entire bin name, if it contains no delimiter) are the same
		{
			free(strFirstPart); // free memory allocated by ExtractFirstPart()
			return true;
		}else{
			free(strFirstPart); // free memory allocated by ExtractFirstPart()
			return false;
		}
	}
}





// ---------------------------------------------------------------------------------
//		GETBININDEX FUNCTION
// ---------------------------------------------------------------------------------
//	This function looks for a media bin name containing the 'name' input parameter string in it, according to the actor's variables. Those must be already set.
//	When a bin is found, return the bin index,
//	else return -1.
//

static int
GetBinIndex(
	IsadoraParameters*	ip,
	ActorInfo*			inActorInfo,
	PluginInfo*			info)
{

	// BROWSE ALL BINS: IF A BIN NAME SATISFIES THE REQUIREMENTS BELOW, RETURN THE BIN INDEX
	// ELSE RETURN -1
	//
	// Requirements:
	// - if delimited == false
	//		bin name == 'name' input parameter
	// - if delimited == true AND delimiter exists in bin name
	//		1st part of bin name == 'name' input property
	// - if delimited == true AND doesn't exist in bin name
	//		bin name == 'name' input property
	//
	// Note: the Bin Index is one-based

	char loopCurrentBinName[BIN_NAME_MAX_LENGTH];
	UInt32 binUniqueID;
	UInt32 mediaCount;
	Boolean binExists;

	int i = 1; // bin index (one-based)

	do
	{
		
		binExists = GetInfoForIndexedBin_(ip, inActorInfo, info->mMediaType, i, loopCurrentBinName, &binUniqueID, &mediaCount); // binUniqueID and mediaCount are unused

		if (binExists == TRUE)
		{
			if (FindNameParameterInBinName(info->mName->strData, loopCurrentBinName, info->mDelimited, info->mDelimiter))
			{
				// Return bin index
				return i;
			}
		}
		
		i++;

	}while (binExists == TRUE); // Loop while a bin is found in bin list

	return -1;
}





// ---------------------------------------------------------------------------------
//		 HandlePropertyChangeValue	[INTERRUPT SAFE]
// ---------------------------------------------------------------------------------
//	### This function is called whenever one of the input values of an actor changes.
//	The one-based property index of the input is given by inPropertyIndex1.
//	The new value is given by inNewValue, the previous value by inOldValue.
//
static void
HandlePropertyChangeValue(
	IsadoraParameters*	ip,
	ActorInfo*			inActorInfo,
	PropertyIndex		inPropertyIndex1,			// the one-based index of the property than changed values
	ValuePtr			/* inOldValue */,			// the property's old value
	ValuePtr			inNewValue,					// the property's new value
	Boolean				/* inInitializing */)		// true if the value is being set when an actor is first initalized
{
	PluginInfo* info = GetPluginInfo_(inActorInfo);

	// ### When you add/change/remove properties, you will need to add cases
	// to this switch statement, to process the messages for your
	// input properties
	
	// The value comes to you encapsulated in a Value structure. See 
	// ValueCommon.h for details about the contents of this structure.
	
	switch (inPropertyIndex1) {
	case kInputMediaType:

		switch (inNewValue->u.ivalue)
		{
		case MEDIA_TYPE_VIDEO:
			info->mMediaType = kMediaTypeMovie;
			break;
		case MEDIA_TYPE_AUDIO:
			info->mMediaType = kMediaTypeAudio;
			break;
		case MEDIA_TYPE_MIDI:
			info->mMediaType = kMediaTypeMidi;
			break;
		case MEDIA_TYPE_PICTURE:
			info->mMediaType = kMediaTypePicture;
			break;
		case MEDIA_TYPE_3D:
			info->mMediaType = kMediaType3D;
			break;
		}


		break;


	case kInputName:
		{
			
			// name property has changed
			info->mName = inNewValue->u.str;

			// remove all forbidden characters ('\r', '\n', '\t')
			removeCharInString ('\r', info->mName->strData);
			removeCharInString ('\n', info->mName->strData);
			removeCharInString ('\t', info->mName->strData);
			
		}
		break;

	case kInputDelimited:
		{
			// delimited property has changed
		if (inNewValue->u.ivalue == 0)
			info->mDelimited = false;
		else
			info->mDelimited = true;
		}
		break;

	case kInputDelimiter:
		{
			// delimiter property has changed
			info->mDelimiter = inNewValue->u.ivalue;
		}
		break;

	case kInputTrigger:
		{
			
			// TRIGGER HAS BEEN FIRED

			// look for a bin name that matches the input properties: if such a bin is found, update output properties
			int binLookedFor = GetBinIndex(ip, inActorInfo, info);

			if (binLookedFor >= 1)
			{

				Value kOutTriggerValue = { kBoolean, 1 };
				Value kOutBinIndexValue = { kInteger, binLookedFor };

				// If bin has been found, update output properties
				SetOutputPropertyValue_(ip, info->mActorInfoPtr, kOutputBinNumber, &kOutBinIndexValue); // output bin index
				SetOutputPropertyValue_(ip, info->mActorInfoPtr, kOutputBinFound, &kOutTriggerValue); // output 'bin found' trigger

			}

			

		}
		
		break;

	}

}


// ---------------------------------------------------------------------------------
//		 GetActorDefinedArea
// ---------------------------------------------------------------------------------
//	If the mGetActorDefinedAreaProc in the ActorInfo struct points to this function,
//	it indicates to Isadora that the object would like to draw either an icon or else
//	an graphic representation of its function.
//
//	### This function uses the 'PICT' 0 resource stored with the plugin to draw an icon.
//  You should replace this picture (located in the Plugin Resources.rsrc file) with
//  the icon for your actor.
// 
static ActorPictInfo	gPictInfo = { false, nil, nil, 0, 0 };

static Boolean
GetActorDefinedArea(
	IsadoraParameters*			ip,			
	ActorInfo*					inActorInfo,
	SInt16*						outTopAreaWidth,			// returns the width to reserve for the top Actor Defined Area
	SInt16*						outTopAreaMinHeight,		// returns the minimum height of the top area
	SInt16*						outBotAreaHeight,			// returns the width to reserve for the bottom Actor Defined Area
	SInt16*						outBotAreaMinWidth)			// returns the minimum width of the bottom area
{
	if (!gPictInfo.mInitialized) {
		PrepareActorDefinedAreaPict_(ip, inActorInfo, 0, &gPictInfo);
	}
	
	// place picture in top area
	*outTopAreaWidth = gPictInfo.mWidth;
	*outTopAreaMinHeight = gPictInfo.mHeight;
	
	// don't draw anything in bottom area
	*outBotAreaHeight = 0;
	*outBotAreaMinWidth = 0;
	
	return true;
}

// ---------------------------------------------------------------------------------
//		 DrawActorDefinedArea
// ---------------------------------------------------------------------------------
//	If GetActorDefinedArea is defined, then this function will be called whenever
//	your ActorDefinedArea needs to be drawn.
//
//	Beacuse we are using the PICT 0 resource stored with this plugin, we can use
//	the DrawActorDefinedAreaPict_ supplied by the Isadora callbacks.
//
//  DrawActorDefinedAreaPict_ is Alpha Channel aware, so you can have nice
//	shading if you like.

static void
DrawActorDefinedArea(
	IsadoraParameters*			ip,
	ActorInfo*					inActorInfo,
	void*						/* inDrawingContext */,		// unused at present
	ActorDefinedAreaPart		inActorDefinedAreaPart,		// the part of the actor that needs to be drawn
	ActorAreaDrawFlagsT			/* inAreaDrawFlags */,		// actor draw flags
	Rect*						inADAArea,					// rect enclosing the entire Actor Defined Area
	Rect*						/* inUpdateArea */,			// subset of inADAArea that needs updating
	Boolean						inSelected)					// TRUE if actor is currently selected
{
	if (inActorDefinedAreaPart == kActorDefinedAreaTop && gPictInfo.mInitialized) {
		DrawActorDefinedAreaPict_(ip, inActorInfo, inSelected, inADAArea, &gPictInfo);
	}
}
	
// ---------------------------------------------------------------------------------
//		 GetActorInfo
// ---------------------------------------------------------------------------------
//	This is function is called by to get the actor's class and ID, and to get
//	pointers to the all of the plugin functions declared locally.
//
//	All members of the ActorInfo struct pointed to by outActorParams have been
//	set to 0 on entry. You only need set functions defined by your plugin
//	
EXPORT_ void
GetActorInfo(
	void*				/* inParam */,
	ActorInfo*			outActorParams)
{
	// REQUIRED information
	outActorParams->mActorName							= kActorName;
	outActorParams->mClass								= kActorClass;
	outActorParams->mID									= kActorID;
	outActorParams->mCompatibleWithVersion				= kCurrentIsadoraCallbackVersion;
	
	// REQUIRED functions
	outActorParams->mGetActorParameterStringProc		= GetParameterString;
	outActorParams->mGetActorHelpStringProc				= GetHelpString;
	outActorParams->mCreateActorProc					= CreateActor;
	outActorParams->mDisposeActorProc					= DisposeActor;
	outActorParams->mActivateActorProc					= ActivateActor;
	outActorParams->mHandlePropertyChangeValueProc		= HandlePropertyChangeValue;
	
	// OPTIONAL FUNCTIONS
	outActorParams->mHandlePropertyChangeTypeProc		= NULL;
	outActorParams->mHandlePropertyConnectProc			= NULL;
	outActorParams->mPropertyValueToStringProc		= PropertyValueToString; // For 'media type' input property
	outActorParams->mPropertyStringToValueProc			= NULL;
	outActorParams->mGetActorDefinedAreaProc			= GetActorDefinedArea;
	outActorParams->mDrawActorDefinedAreaProc			= DrawActorDefinedArea;
	outActorParams->mMouseTrackInActorDefinedAreaProc	= NULL;
}



// ---------------------------------------------------------------------------------
//		 ReceiveMessage
// ---------------------------------------------------------------------------------
//	Isadora broadcasts messages to its Message Receives depending on what message
//	they are listening to. In this case, we are listening for kWantVideoFrameTick,
//	which is broadcast periodically (30 times per second.) When we receive the
//	message, we check to see if our video frame needs to be updated. If so, we
//	process the incoming video and pass the newly generated frame to the output.

static void
ReceiveMessage(
	IsadoraParameters*	ip,
	MessageMask			/* inMessageMask */,		// the message that caused this ReceiveMessage to be called -- one of the kWant... constants
	PortIndex			/* inIndex1 */,				// for MIDI messages, the port from which the message arrived. n/a otherwise
	const MsgData*		/* inData */,				// the data associated with this message
	UInt32				/* inLen */,				// the length of the data associated with this message
	long				inRefCon)					// in our use, actually the pointer to our ActorInfo
{
	//// Convert the refCon into the ActorInfo* that it
	//// really is, so that we can get at our data
	//ActorInfo* actorInfo = reinterpret_cast<ActorInfo*>(inRefCon);

	//// get pointer to plugin info
	//PluginInfo* info = GetPluginInfo_(actorInfo);

	//// We use this Value struct in a few places below...
	//Value v = { kData, nil };

	//// set a flag to remember if we had an output buffer before we
	//// called UpdateImageBufferMap
	//Boolean wasOutputBuffer = info->mImageBufferMap.mOutputBuffersValid;
	//
	//// ensure that the ImageBufferMap is up valid for the
	//// current input Image Buffer
	//UpdateImageBufferMap(ip, &info->mImageBufferMap);
	//
	//// use GetImageBufferPtr to get the input and output buffers
	//ImageBufferPtr img1 = GetImageBufferPtr(ip, &info->mImageBufferMap, 0);
	//ImageBufferPtr out = GetOutputImageBufferPtr(&info->mImageBufferMap, 0);

	//// if we don't have a valid output buffer

	//if (info->mImageBufferMap.mOutputBuffersValid == false) {
	//	
	//	// if there was an output buffer preivously, we need to 
	//	// send a 'nil' buffer to let other modules know that 
	//	// our ouptut is now invalid
	//	if (wasOutputBuffer) {
	//		v.u.data = nil;
	//		SetOutputPropertyValue_(ip, actorInfo, kOutputVideo, &v);
	//	}

	//// otherwise, if our mNeedsDraw flag is true, and if we have both
	//// and input buffer and an output buffer, then we can proceed to 
	//// process the video image
	//// 
	//// we only draw the image if the following are true:
	//// 1) the mNeedsDraw variable is set to true (this is set in the
	////    InputPropertyChangeValue callback above.)
	//// 2) the input image buffer (img1) is not nil
	//// 3) the output image buffer (out) is not nil
	//
	//} else if (info->mNeedsDraw && img1 != nil && out != nil) {

	//	// call EnterVideoProcessing_ so that Isadora will accumulate the
	//	// amount of time spent processing the video data - this is not
	//	// requried by highly recommended so that the VPO value in the
	//	// Status Window stays accurate.
	//	UInt64 vpStart = EnterVideoProcessing_(ip);
	//	
	//	// clear the mNeedsDraw flag
	//	info->mNeedsDraw = false;
	//
	//	// assume for the moment that we won't draw the frame
	//	// set this value to true if we change the output
	//	Boolean drawFrame = false;
	//	
	//	// we only process 32 bit data in this plugin
	//	if (out->mBitDepth == 32) {
	//		ProcessVideoFrame(ip, info, img1, out);
	//		drawFrame = true;
	//	}
	//	
	//	// if the drawFrame flag got set, then we need to output the
	//	// new video data to our output port here.
	//	
	//	if (drawFrame) {
	//		
	//		// IMPORTANT: We have changed the data in the output buffer
	//		// so we need to increment the data change count so that
	//		// those looking at our data will know that there is new
	//		// data in the buffer
	//		
	//		out->mInfo.mDataChangeCount++;
	//		v.u.data = out;
	//		
	//		// send the new video frame to the video output property
	//		SetOutputPropertyValue_(ip, info->mActorInfoPtr, kOutputVideo, &v);
	//	}
	//	
	//	// make sure to compliment EnterVideoProcessing_ with 
	//	// and ExitVideoProcessing_ call
	//	ExitVideoProcessing_(ip, vpStart);
	//}
}
